/**
 * Define api of apeman-api-format
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aglob = require('aglob')
const execcli = require('execcli')
const { statAsync } = require('asfs')
const fmtjson = require('fmtjson')
const apemanApiPath = require('apeman-api-path')
const load = require('apeman-api-client/load')
const aslogger = require('aslogger')
const spec = require('./spec')

let cwd = () => process.cwd()

/** @lends define */
function define (options = {}) {
  let { logging = false } = options
  let logger = aslogger({ disabled: !logging })
  let { root = cwd() } = options

  let pathApi = apemanApiPath({ root, logging: false })

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Format json files.
     * @method formatJson
     * @param {string} pattern - Glob filename patterns
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    formatJson (pattern, options = {}) {
      const s = this
      return co(function * () {
        let path = yield load(pathApi, { $$ctx: s })
        options.cwd = yield path.resolve(options.cwd || cwd())
        yield fmtjson(pattern, options)
      })
    },

    /**
     * Format js files.
     * @method formatJs
     * @param {string} pattern - Filename pattern
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    formatJs (pattern, options = {}) {
      return co(function * () {
        let filenames = yield aglob(pattern, {
          ignore: options.ignore
        })
        for (let filename of filenames) {
          let before = yield statAsync(filename).catch(() => null)
          yield execcli('standard-format', [ '-w', filename ], {
            notfound: 'Try `npm install -g standard-format`'
          })
          let after = yield statAsync(filename).catch(() => null)
          let changed = before.size !== after.size
          if (changed) {
            logger.debug(`File formatted: ${filename}`)
          }
        }
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
